==> crossroads-0.11.tar.gz: crossroads-0.11/src/choosebackend.c <==
#include "crossroads.h"

void choose_backend () {
    int backends[MAX_BACKEND], nbackends = 0, i;
    unsigned long long values[MAX_BACKEND];
    unsigned long long nbytes;
    int nbytes_set = 0;

    /* Populate the array of selectable backends. */
    for (i = 0; i < activeservice->nbackend; i++)
	if (servicereport->backendstate[i].avail == st_available)
	    backends[nbackends++] = i;

    /* No backends? Nogo. */
    if (!nbackends) {
	current_backend = -1;
	warning ("No active backends to select!\n");
	return;
    }

    /* Only one backend? Then it's always the first one,
     * whatever you do.
     */
    if (nbackends == 1) {
	current_backend = backends[0];
	msg ("Only 1 backend to select (%d)\n", current_backend);
	return;
    }

    /* More than 1 backend available.. make a wise choice. */
    switch (activeservice->dispatchtype) {
	
    case ds_roundrobin:
	/* Find the currently used backend, go one forward. */
	for (i = 0; i < nbackends; i++)
	    if (backends[i] == current_backend) {
		i++;
		i %= nbackends;
		current_backend = backends[i];
		msg ("Chosen backend (roundrobin): %d\n", current_backend);
		return;
	    }
	/* None found.. try the first one. */
	current_backend = backends[0];
	return;
	
    case ds_random:
	/* Choose a random one of the availables. */
	current_backend = backends[rand() % nbackends];
	msg ("Chosen backend (random): %d\n", current_backend);
	return;

    case ds_bysize:
	/* Fill the values */
	for (i = 0; i < nbackends; i++)
	    if (activeservice->dispatchover)
		values[i] =
		    servicereport->backendstate[backends[i]].avg_nbytes *
		    activeservice->backend[backends[i]].weight;
	    else
		values[i] = servicereport->backendstate[backends[i]].nbytes *
		    activeservice->backend[backends[i]].weight;
	/* Get the backend that has transported the least bytes */
	for (i = 0; i < nbackends; i++)
	    if (!nbytes_set++ || values[i] < nbytes) {
		nbytes = values[i];
		current_backend = backends[i];
	    }
	msg ("Chosen backend (by size): %d\n", current_backend);
	return;

    case ds_byduration:
	/* Fill the values */
	for (i = 0; i < nbackends; i++)
	    if (activeservice->dispatchover)
		values[i] =
		    servicereport->backendstate[backends[i]].avg_nsec *
		    activeservice->backend[backends[i]].weight;
	    else
		values[i] = servicereport->backendstate[backends[i]].nsec *
		    activeservice->backend[backends[i]].weight;
	/* Get the backend that has transported the least connection secs */
	for (i = 0; i < nbackends; i++)
	    if (!nbytes_set++ || values[i] < nbytes) {
		nbytes = values[i];
		current_backend = backends[i];
	    }
	msg ("Chosen backend (by duration): %d\n", current_backend);
	return;

    case ds_byorder:
	/* Get the first available back end in line. */
	current_backend = backends[0];
	msg ("Chosen backend (by order): %d\n", current_backend);
	return;
	
    default:
	/* Internal fry.. */	
	error ("Internal error: unhandled dispatch type %d\n",
	       activeservice->dispatchtype);
    }
}

==> crossroads-0.17.tar.gz: crossroads-0.17/src/choosebackend.c <==
#include "crossroads.h"

void choose_backend () {
    int backends[MAX_BACKEND], nbackends = 0, i;
    unsigned long long values[MAX_BACKEND];
    unsigned long long nbytes;
    int nbytes_set = 0;

    /* Populate the array of selectable backends. */
    for (i = 0; i < activeservice->nbackend; i++) {
	/* Backend i is a candidate if:
	 * (a) it is available, AND
	 * (b) there's no max of sessions, or the actual nr of sessions
	 *     is below max
	 */
	if (servicereport->backendstate[i].avail == st_available &&
	    (activeservice->backend[i].maxclients == 0 ||
	     activeservice->backend[i].maxclients >
	     servicereport->backendstate[i].nclients))
	    backends[nbackends++] = i;
    }
    
    /* No backends? Nogo. */
    if (!nbackends) {
	current_backend = -1;
	warning ("No active backends to select!\n");
	return;
    }

    /* Only one backend? Then it's always the first one,
     * whatever you do.
     */
    if (nbackends == 1) {
	current_backend = backends[0];
	msg ("Only 1 backend to select (%d)\n", current_backend);
	return;
    }

    /* More than 1 backend available.. make a wise choice. */
    switch (activeservice->dispatchtype) {
	
    case ds_roundrobin:
	/* Find the currently used backend, go one forward. */
	for (i = 0; i < nbackends; i++)
	    if (backends[i] == current_backend) {
		i++;
		i %= nbackends;
		current_backend = backends[i];
		msg ("Chosen backend (roundrobin): %d\n", current_backend);
		return;
	    }
	/* None found.. try the first one (run  to momma). */
	current_backend = backends[0];
	return;
	
    case ds_random:
	/* Choose a random one of the availables. */
	current_backend = backends[rand() % nbackends];
	msg ("Chosen backend (random): %d\n", current_backend);
	return;

    case ds_bysize:
	/* Fill the values */
	for (i = 0; i < nbackends; i++)
	    if (activeservice->dispatchover)
		values[i] =
		    servicereport->backendstate[backends[i]].avg_nbytes *
		    activeservice->backend[backends[i]].weight;
	    else
		values[i] = servicereport->backendstate[backends[i]].nbytes *
		    activeservice->backend[backends[i]].weight;
	/* Get the backend that has transported the least bytes */
	for (i = 0; i < nbackends; i++)
	    if (!nbytes_set++ || values[i] < nbytes) {
		nbytes = values[i];
		current_backend = backends[i];
	    }
	msg ("Chosen backend (by size): %d\n", current_backend);
	return;

    case ds_byduration:
	/* Fill the values */
	for (i = 0; i < nbackends; i++)
	    if (activeservice->dispatchover)
		values[i] =
		    servicereport->backendstate[backends[i]].avg_nsec *
		    activeservice->backend[backends[i]].weight;
	    else
		values[i] = servicereport->backendstate[backends[i]].nsec *
		    activeservice->backend[backends[i]].weight;
	/* Get the backend that has transported the least connection secs */
	for (i = 0; i < nbackends; i++)
	    if (!nbytes_set++ || values[i] < nbytes) {
		nbytes = values[i];
		current_backend = backends[i];
	    }
	msg ("Chosen backend (by duration): %d\n", current_backend);
	return;

    case ds_byorder:
	/* Get the first available back end in line. */
	current_backend = backends[0];
	msg ("Chosen backend (by order): %d\n", current_backend);
	return;
	
    default:
	/* Internal fry.. */	
	error ("Internal error: unhandled dispatch type %d\n",
	       activeservice->dispatchtype);
    }
}
